package fse.eclipse.branchrecorder.util;

import java.io.File;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import java.util.StringTokenizer;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.preferences.IEclipsePreferences;
import org.eclipse.core.runtime.preferences.InstanceScope;
import org.eclipse.core.variables.IStringVariableManager;
import org.eclipse.core.variables.VariablesPlugin;
import org.eclipse.jgit.lib.Repository;
import org.eclipse.jgit.util.FS;
import org.jtool.editrecorder.util.WorkspaceUtilities;

/**
 * Original source code is {@link org.eclipse.egit.core.RepositoryUtil}
 */
public class RepositoryUtil {
    private static final IEclipsePreferences prefs = InstanceScope.INSTANCE.getNode("org.eclipse.egit.core");
    private static final String PREFS_DIRECTORIES = "GitRepositoriesView.GitDirectories";
    private static final String PREFS_DIRECTORIES_REL = "GitRepositoriesView.GitDirectories.relative";
    private static final String PREFS_DEFAULT_REPO_KEY = "core_defaultRepositoryDir";
    private static final String EGIT_PLUGIN_ID = "org.eclipse.egit.core";

    public static IProject convertIProject(Repository repository) {
        // IWorkspaceRoot#getProject() では拾えきれない場合がある
        IWorkspaceRoot workspace = WorkspaceUtilities.getWorkspace().getRoot();

        String path = repository.getDirectory().getParent();
        for (IProject p : workspace.getProjects()) {
            String path2 = p.getLocation().toOSString();
            if (PathUtil.isEqualPath(path, path2)) {
                return p;
            }
        }
        return null;
    }

    public static String getDefaultRepositoryDir() {
        String key = PREFS_DEFAULT_REPO_KEY;
        String dir = migrateRepoRootPreference();
        IEclipsePreferences p = InstanceScope.INSTANCE.getNode(EGIT_PLUGIN_ID);
        if (dir == null) {
            dir = p.get(key, getDefaultDefaultRepositoryDir());
        } else {
            p.put(key, dir);
        }
        IStringVariableManager manager = VariablesPlugin.getDefault().getStringVariableManager();
        String result;
        try {
            result = manager.performStringSubstitution(dir);
        } catch (CoreException e) {
            result = ""; //$NON-NLS-1$
        }
        if (result == null || result.isEmpty()) {
            result = ResourcesPlugin.getWorkspace().getRoot().getRawLocation().toOSString();
        }
        return result;
    }

    private static String getDefaultDefaultRepositoryDir() {
        return new File(FS.DETECTED.userHome(), "git").getPath(); //$NON-NLS-1$
    }

    private static String migrateRepoRootPreference() {
        IEclipsePreferences p = InstanceScope.INSTANCE.getNode("org.eclipse.egit.ui"); //$NON-NLS-1$
        String deprecatedUiKey = "default_repository_dir"; //$NON-NLS-1$
        String value = p.get(deprecatedUiKey, null);
        if (value != null && value.isEmpty()) {
            value = null;
        }
        if (value != null) {
            p.remove(deprecatedUiKey);
        }
        return value;
    }

    public static Set<String> getRepositoryPaths() {
        String dirString;
        Set<String> dirs;
        synchronized (prefs) {
            dirString = prefs.get(PREFS_DIRECTORIES_REL, ""); //$NON-NLS-1$
            if (dirString.equals("")) { //$NON-NLS-1$
                dirs = migrateAbolutePaths();
            } else {
                dirs = toDirSet(dirString);
            }
        }
        return dirs;
    }

    private static Set<String> migrateAbolutePaths() {
        String dirString;
        Set<String> dirs;
        dirString = prefs.get(PREFS_DIRECTORIES, ""); //$NON-NLS-1$
        dirs = toDirSet(dirString);
        return dirs;
    }

    private static Set<String> toDirSet(String dirs) {
        if (dirs == null || dirs.isEmpty()) {
            return Collections.emptySet();
        }
        Set<String> configuredStrings = new HashSet<String>();
        StringTokenizer tok = new StringTokenizer(dirs, File.pathSeparator);
        while (tok.hasMoreTokens()) {
            configuredStrings.add(ResourcesPlugin.getWorkspace().getRoot().getLocation().toFile().toPath().resolve(tok.nextToken()).toString());
        }
        return configuredStrings;
    }
}
