package fse.eclipse.mergehelper.detector;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.ResourcesPlugin;
import org.jtool.changerecorder.history.OperationHistory;
import org.jtool.changerecorder.history.XmlConstantStrings;
import org.jtool.changerecorder.util.XmlFileStream;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import fse.eclipse.branchrecorder.commit.history.MH_Operation2Xml;
import fse.eclipse.branchrecorder.commit.history.MH_Xml2Operation;
import fse.eclipse.branchrecorder.commit.history.XmlCommitWriter;
import fse.eclipse.branchrecorder.commit.operation.CommitOperation;
import fse.eclipse.mergehelper.Activator;
import fse.eclipse.mergehelper.element.BranchRootInfo;
import fse.eclipse.mergehelper.element.MergeType;
import fse.eclipse.mergehelper.util.FileReader;
import fse.eclipse.mergehelper.util.ProjectUtil;

public class BranchHistoryCopier extends AbstractDetector {
    private static final String TASK_NAME = "Copy History";
    private static final String ERROR_MESSAGE = "NULL";
    private static AbstractDetector instance = new BranchHistoryCopier();

    public static final String BRANCH_NAME_MARK = "@";
    private Holder a_holder;
    private Holder j_holder;

    private BranchHistoryCopier() {
    }

    public static AbstractDetector getInstance() {
        return instance;
    }

    @Override
    public void execute() {
        BranchRootInfo rootInfo = BranchRootInfo.getInstance();
        CommitOperation baseCop = searchBaseCommitOperation(rootInfo);
        copy(baseCop, rootInfo);
    }

    private CommitOperation searchBaseCommitOperation(BranchRootInfo rootInfo) {
        StringBuilder sb = new StringBuilder();
        String branchName = rootInfo.getBranchName(MergeType.ACCEPT);
        sb.append(ProjectUtil.getProjectHistoryPath(rootInfo.getProject()));
        sb.append(File.separator).append(branchName);
        String a_historyPath = sb.toString();

        int length = sb.length();
        sb.delete(length - branchName.length(), length);
        branchName = rootInfo.getBranchName(MergeType.JOIN);
        sb.append(branchName);
        String j_historyPath = sb.toString();

        List<CommitOperation> a_cops = new ArrayList<>();
        List<CommitOperation> j_cops = new ArrayList<>();

        a_holder = new Holder(FileReader.getAllXmlFiles(a_historyPath));
        j_holder = new Holder(FileReader.getAllXmlFiles(j_historyPath));

        while (true) {
            CommitOperation a_cop = searchNextCommitOperation(a_holder);
            if (a_cop != null) {
                a_cops.add(a_cop);
                CommitOperation baseCop = searchBaseCommitOperation(a_cops, j_cops);
                if (baseCop != null) {
                    return baseCop;
                }
            }

            CommitOperation j_cop = searchNextCommitOperation(j_holder);
            if (j_cop != null) {
                j_cops.add(j_cop);
                CommitOperation baseCop = searchBaseCommitOperation(a_cops, j_cops);
                if (baseCop != null) {
                    return baseCop;
                }
            }

            if (a_cop == null && j_cop == null) {
                break;
            }
        }
        return null;
    }

    private CommitOperation searchBaseCommitOperation(List<CommitOperation> a_cops, List<CommitOperation> j_cops) {
        int a_size = a_cops.size();
        int j_size = j_cops.size();

        // Join:parentId Accept:commitIdから検索
        // (2つのブランチにXMLCommitFileが1つしかないとき kuso仕様)
        for (int j = 0; j < j_size; j++) {
            CommitOperation j_cop = j_cops.get(j);
            String j_id = j_cop.getCommitId();
            String j_pid = j_cop.getParentCommitId();
            for (int a = 0; a < a_size; a++) {
                CommitOperation a_cop = a_cops.get(a);
                String a_id = a_cop.getCommitId();
                String a_pid = a_cop.getParentCommitId();
                if (a_id.equals(j_pid)) {
                    return a_cop;
                } else if (j_id.equals(a_pid)) {
                    return j_cop;
                }
            }
        }
        return null;
    }

    private CommitOperation searchNextCommitOperation(Holder holder) {
        for (int i = holder.index - 1; i >= 0; i--) {
            File file = holder.files.get(i);
            if (file.getName().endsWith(XmlCommitWriter.CommitXmlFileName)) {
                holder.index = i;
                return convertCommitOperation(file);
            }
        }
        return null;
    }

    private void copy(CommitOperation cop, BranchRootInfo rootInfo) {
        String branchName = rootInfo.getBranchInfo(MergeType.ACCEPT).getName();
        long time = 0;
        if (cop != null) {
            time = cop.getTime();
        }
        int a = 0, j = 0;
        for (int i = a_holder.files.size() - 1; i >= 0; i--) {
            a++;
            File file = a_holder.files.get(i);
            OperationHistory history = readOperationHistory(file);
            if (history != null && history.size() > 0 && history.getFirstOperation().getTime() >= time) {
                Document doc = MH_Operation2Xml.convert(history);
                Document renameDoc = renameFileAttr(doc, branchName);
                copy(renameDoc, branchName, file);
            }
        }

        branchName = rootInfo.getBranchInfo(MergeType.JOIN).getName();
        for (int i = j_holder.files.size() - 1; i >= 0; i--) {
            j++;
            File file = j_holder.files.get(i);
            OperationHistory history = readOperationHistory(file);
            if (history != null && history.size() > 0 && history.getFirstOperation().getTime() >= time) {
                Document doc = MH_Operation2Xml.convert(history);
                Document renameDoc = renameFileAttr(doc, branchName);
                copy(renameDoc, branchName, file);
            }
        }
        System.out.println("a:" + a + "   j:" + j);
    }

    private void copy(Document doc, String name, File file) {
        StringBuilder sb = new StringBuilder();
        sb.append(Activator.getWorkingDirPath()).append(File.separator);
        sb.append(name).append(File.separator);

        String abpath = file.getAbsolutePath();
        int idx = abpath.indexOf(name);
        if (idx != -1) {
            sb.append(abpath.substring(idx + name.length()));
        } else {
            sb.append(file.getName());
        }
        XmlFileStream.write(doc, sb.toString(), ResourcesPlugin.getEncoding());
    }

    private OperationHistory readOperationHistory(File file) {
        Document doc = XmlFileStream.read(file.getAbsolutePath());
        OperationHistory history = MH_Xml2Operation.convert(doc);
        history.sort();
        return history;
    }

    private Document renameFileAttr(Document doc, String name) {
        // operation まで下がる
        NodeList nodeList = doc.getFirstChild().getChildNodes();
        int length = nodeList.getLength();
        int i;
        for (i = 0; i < length; i++) {
            if (nodeList.item(i).getNodeName().equals(XmlConstantStrings.OperationsElem)) {
                break;
            }
        }
        return renameFileAttr(doc, nodeList.item(i).getChildNodes(), name);
    }

    private Document renameFileAttr(Document doc, NodeList nodeList, String name) {
        // fileAttributeプロジェクト名の部分にブランチ名を足す
        int length = nodeList.getLength();
        for (int i = 0; i < length; i++) {
            Element e = (Element) nodeList.item(i);
            String fileAttr = e.getAttribute(XmlConstantStrings.FileAttr);
            if (!e.getNodeName().equals(XmlConstantStrings.CompoundOperationElem)) {
                if (!fileAttr.equals("null") && fileAttr.length() != 0) {
                    fileAttr = renameFileAttr(fileAttr, name);
                    e.setAttribute(XmlConstantStrings.FileAttr, fileAttr);
                }
            } else {
                renameFileAttr(doc, e.getChildNodes(), name);
            }
        }
        return doc;
    }

    private String renameFileAttr(String fileAttr, String name) {
        int idx = fileAttr.indexOf("/", 1);
        if (idx == -1) {
            idx = fileAttr.indexOf("\\", 1);
        }

        StringBuilder sb = new StringBuilder(fileAttr);
        sb.insert(idx, BRANCH_NAME_MARK);
        sb.insert(idx + BRANCH_NAME_MARK.length(), name);
        return sb.toString();
    }

    private CommitOperation convertCommitOperation(File file) {
        OperationHistory history = readOperationHistory(file);
        return (CommitOperation) history.getFirstOperation();
    }

    private class Holder {
        List<File> files;
        int index;

        Holder(List<File> files) {
            this.files = files;
            index = files.size();
        }
    }

    @Override
    protected String getTaskName() {
        return TASK_NAME;
    }

    @Override
    protected String getErrorMessage() {
        return ERROR_MESSAGE;
    }

    @Override
    protected void nextState() {
        HistoryNormalizer.getInstance().detect(dialog);
    }
}
