package fse.eclipse.mergehelper.detector.merge;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.jtool.changerepository.data.FileInfo;
import org.jtool.changerepository.data.ProjectInfo;
import org.jtool.changerepository.operation.CodeInsertedOperation;
import org.jtool.changerepository.operation.UnifiedOperation;
import org.jtool.changerepository.parser.OpJavaElement;

import fse.eclipse.mergehelper.detector.AbstractDetector;
import fse.eclipse.mergehelper.detector.merge.AMerge.MergeIndexHolder;
import fse.eclipse.mergehelper.element.BranchFileInfo;
import fse.eclipse.mergehelper.element.BranchInfo;
import fse.eclipse.mergehelper.element.BranchJavaElement;
import fse.eclipse.mergehelper.element.BranchRootInfo;
import fse.eclipse.mergehelper.element.MergePoint;
import fse.eclipse.mergehelper.element.MergeType;
import fse.eclipse.mergehelper.ui.dialog.ConflictDetectingDialog;
import fse.eclipse.mergehelper.util.Parser;

public class AMergedAdjustOperationOffset extends AbstractDetector {
    private static final String TASK_NAME = "Ajust Operation Offset";
    private static final String ERROR_MESSAGE = "Ajust Offset Failed";
    private static AMergedAdjustOperationOffset instance = new AMergedAdjustOperationOffset();

    private boolean isSuccess = true;
    private List<MergeIndexHolder> mergeIdxHoloders;

    private AMergedAdjustOperationOffset() {
    }

    public static AMergedAdjustOperationOffset getInstance() {
        return instance;
    }

    public void detect(ConflictDetectingDialog dialog, List<MergeIndexHolder> mergeIdxHoloders) {
        this.mergeIdxHoloders = mergeIdxHoloders;
        super.detect(dialog);
    }

    @Override
    protected void execute() {
        if (mergeIdxHoloders == null) {
            throw new Error();
        }

        BranchRootInfo rootInfo = BranchRootInfo.getInstance();
        BranchInfo a_bInfo = rootInfo.getBranchInfo(MergeType.ACCEPT);
        BranchInfo j_bInfo = rootInfo.getBranchInfo(MergeType.JOIN);

        initBranchFileInfoMergedResult(a_bInfo);
        initBranchFileInfoMergedResult(j_bInfo);

        MergePoint mPoint = rootInfo.getMergePoint();
        try {
            adjustOffset(a_bInfo, mPoint);
        } catch (NullPointerException e) {
            e.printStackTrace();
        }

        try {
            adjustOffset(j_bInfo, mPoint);
        } catch (NullPointerException e) {
            e.printStackTrace();
        }
    }

    private void initBranchFileInfoMergedResult(BranchInfo bInfo) {
        String branchName = bInfo.getName();
        List<FileInfo> fInfos = bInfo.getProjectInfo().getAllFileInfo();
        for (FileInfo fInfo : fInfos) {
            String code = AMerge.getAMergedCode(fInfo, branchName);

            int size = mergeIdxHoloders.size();
            for (int i = 0; i < size; i++) {
                MergeIndexHolder holder = mergeIdxHoloders.get(i);
                if (holder.equalsFileInfo(fInfo)) {
                    BranchFileInfo bfInfo = bInfo.getBranchFileInfo(fInfo);
                    if (bfInfo != null) {
                        int insIdx = holder.getIndex() + 1;
                        bfInfo.setMergedResult(insIdx, code);
                        mergeIdxHoloders.remove(i);
                    }
                    break;
                }
            }
        }
    }

    private void adjustOffset(BranchInfo bInfo, MergePoint mPoint) throws NullPointerException {
        ProjectInfo pInfo = bInfo.getProjectInfo();
        List<UnifiedOperation> ops = pInfo.getOperations();
        int size = ops.size();

        MergeType type = bInfo.getType();
        UnifiedOperation point = mPoint.getReferenceOperation(type);
        int mIdx = point.indexOfProjectInfo();

        Map<String, Map<Integer, String>> fInfoCodeMap = new HashMap<>();
        Map<String, CodeHolder> preCodeMap = new HashMap<>();
        createFileInfoCodeMap(fInfoCodeMap, ops, mIdx);

        insertAMergeOperation(bInfo);

        for (int i = mIdx + 1; i < size; i++) {
            UnifiedOperation op = ops.get(i);
            if (!op.isTextEditOperation()) {
                continue;
            }

            FileInfo fInfo = op.getFileInfo();
            String name = fInfo.getQualifiedName();
            BranchFileInfo bfInfo = bInfo.getBranchFileInfo(fInfo);

            CodeHolder holder;
            if (preCodeMap.containsKey(name)) {
                holder = preCodeMap.get(name);
            } else {
                String preCode = bfInfo.getMergedResultCode();
                List<OpJavaElement> preElems = new ArrayList<>();
                Parser.collectElements(fInfo, preCode, preElems, true);
                holder = new CodeHolder(preCode, preElems);
            }

            // AMergeOperation分 -1
            int idx = op.indexOfFileInfo() - 1;
            String code = selectFileInfoCode(fInfoCodeMap, fInfo, idx);
            if (code == null) {
                continue;
            }

            List<OpJavaElement> elems = new ArrayList<>();
            Parser.collectElements(fInfo, code, elems, true);

            BranchJavaElement targetElem = bfInfo.getBranchJavaElement(op);
            if (targetElem != null) {
                boolean isAdjusted = false;
                for (OpJavaElement elem : elems) {
                    if (elem.getFullName().equals(targetElem.getFullName())) {
                        adjustOperationOffset(op, holder.elems, elems, elem);
                        isAdjusted = true;
                        break;
                    }
                }
                if (!isAdjusted) {
                    // throw new Error();
                }
            } else {
                // フィールド・メソッド以外に関する操作
                OpJavaElement elem = searchNearElement(elems, op.getStart());
                if (elem != null) {
                    adjustOperationOffset(op, holder.elems, elems, elem);
                }
            }

            if (i + 1 != size) {
                holder.code = fInfo.getCode(idx + 1);
                elems = new ArrayList<>();
                Parser.collectElements(fInfo, holder.code, elems, true);
                holder.elems = elems;
                preCodeMap.put(fInfo.getQualifiedName(), holder);
            }
        }

    }

    private void insertAMergeOperation(BranchInfo bInfo) {
        List<BranchFileInfo> bfInfos = bInfo.getAllBranchFileInfo();
        for (BranchFileInfo bfInfo : bfInfos) {
            FileInfo fInfo = bfInfo.getFileInfo();
            List<UnifiedOperation> ops = fInfo.getOperations();

            int insIdx = bfInfo.getMergedOperationIdx();
            String code = bfInfo.getMergedResultCode();
            CodeInsertedOperation ciop = new CodeInsertedOperation(fInfo, code);
            ops.add(insIdx, ciop);
            fInfo.createOperationManager();
        }
    }

    private void createFileInfoCodeMap(Map<String, Map<Integer, String>> fInfoCodeMap, List<UnifiedOperation> ops, int mIdx) {
        int size = ops.size();
        for (int i = mIdx + 1; i < size; i++) {
            UnifiedOperation op = ops.get(i);
            FileInfo fInfo = op.getFileInfo();
            String name = fInfo.getQualifiedName();

            if (!fInfoCodeMap.containsKey(name)) {
                int idx = op.indexOfFileInfo();
                List<UnifiedOperation> fops = fInfo.getOperations();
                int fsize = fops.size();
                Map<Integer, String> codeMap = new ConcurrentHashMap<>();
                for (int j = idx; j < fsize; j++) {
                    UnifiedOperation fop = fops.get(j);
                    if (!fop.isTextChangedOperation()) {
                        continue;
                    }
                    codeMap.put(j, fInfo.getCode(j));
                }
                fInfoCodeMap.put(name, codeMap);
            }
        }
    }

    private String selectFileInfoCode(Map<String, Map<Integer, String>> fInfoCodeMap, FileInfo fInfo, int idx) {
        Map<Integer, String> codeMap = fInfoCodeMap.get(fInfo.getQualifiedName());
        if (codeMap != null) {
            return codeMap.get(idx);
        } else {
            return null;
        }
    }

    private void adjustOperationOffset(UnifiedOperation op, List<OpJavaElement> preElems, List<OpJavaElement> elems, OpJavaElement elem) {
        OpJavaElement preElem = searchEqualNameElement(preElems, elem);
        if (preElem == null) {
            // AMerge時に要素がない (後にadd methodなど)
            elem = searchNearestElement(preElems, elems, op.getStart());
            if (elem != null) {
                preElem = searchEqualNameElement(preElems, elem);
            }
        }
        if (preElem != null && elem != null) {
            adjustOperationOffset(op, preElem, elem);
        }
    }

    private void adjustOperationOffset(UnifiedOperation op, OpJavaElement mergedElem, OpJavaElement elem) {
        int relativeOffset = op.getStart() - elem.getStart();
        int adjustOffset = mergedElem.getStart() + relativeOffset;
        op.setStart(adjustOffset);
    }

    private OpJavaElement searchEqualNameElement(List<OpJavaElement> elems, OpJavaElement elem) {
        for (OpJavaElement e : elems) {
            if (e.getFullName().equals(elem.getFullName())) {
                return e;
            }
        }
        return null;
    }

    private OpJavaElement searchNearElement(List<OpJavaElement> elems, int offset) {
        for (OpJavaElement elem : elems) {
            if (elem.getEnd() < offset) {
                return elem;
            }
        }
        return null;
    }

    private OpJavaElement searchNearestElement(List<OpJavaElement> mergedElems, List<OpJavaElement> currentCodeElems, int offset) {
        List<OpJavaElement> elemHolers = new ArrayList<>();
        for (OpJavaElement mElem : mergedElems) {
            int size = currentCodeElems.size();
            for (int i = 0; i < size; i++) {
                OpJavaElement elem = currentCodeElems.get(i);
                if (!elemHolers.contains(elem) && mElem.getFullName().equals(elem.getFullName())) {
                    elemHolers.add(elem);
                    currentCodeElems.remove(i);
                    break;
                }
            }
        }
        return searchNearElement(elemHolers, offset);
    }

    @Override
    protected String getTaskName() {
        return TASK_NAME;
    }

    @Override
    protected String getErrorMessage() {
        return ERROR_MESSAGE;
    }

    @Override
    protected void nextState() {
        if (isSuccess) {
            finish();
        } else {
            error();
        }
    }

    private class CodeHolder {
        String code;
        List<OpJavaElement> elems;

        CodeHolder(String code, List<OpJavaElement> elems) {
            this.code = code;
            this.elems = elems;
        }
    }

}
