package fse.eclipse.mergehelper.ui;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IPersistableElement;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import org.jtool.changereplayer.ui.ChangeHistoryEditor;
import org.jtool.changereplayer.ui.SourceCodeViewLoader;
import org.jtool.changerepository.data.FileInfo;

import fse.eclipse.mergehelper.Activator;
import fse.eclipse.mergehelper.util.RepositoryElementInfoUtil;

public class MH_ChangeHistoryEditor extends ChangeHistoryEditor {
    public static final String ID = "ChangeReplayer.editor.MH_ChangeHistoryEditor";

    private static MH_ChangeHistoryEditor editor;

    public MH_ChangeHistoryEditor() {
        super();
    }

    public static MH_ChangeHistoryEditor open() {
        if (editor != null) {
            return editor;
        }

        try {
            IWorkbenchPage page = Activator.getWorkbenchPage();
            IEditorPart editorPart = IDE.openEditor(page, new EmptyEditorInput(), ID, true);
            if (editorPart instanceof MH_ChangeHistoryEditor) {
                editor = (MH_ChangeHistoryEditor) editorPart;
            }
        } catch (PartInitException e) {
            e.printStackTrace();
        }
        return editor;
    }

    @Override
    public void show(FileInfo finfo) {
        super.show(finfo);

        String bName = RepositoryElementInfoUtil.getBranchName(finfo);
        StringBuilder sb = new StringBuilder();
        sb.append(bName).append("#").append(finfo.getName());
        setPartName(sb.toString());
    }

    @Override
    public void init(IEditorSite site, IEditorInput input) throws PartInitException {
        setSite(site);
        setInput(input);
        activeView = MH_viewLoader();
        activeView.createBottonActions(site);
    }

    @Override
    public void createPartControl(Composite parent) {
        FillLayout layout = new FillLayout();
        parent.setLayout(layout);

        activeView.createControls(parent);
    }

    @Override
    public void dispose() {
        editor = null;
        super.dispose();
        PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().closeEditor(this, true);
    }

    private MH_SourceCodeView MH_viewLoader() {
        IExtensionRegistry registry = Platform.getExtensionRegistry();
        IExtensionPoint point = registry.getExtensionPoint(SourceCodeViewLoader.EXTENSION_POINT_ID);
        if (point == null) {
            return null;
        }

        IExtension[] extensions = point.getExtensions();
        if (extensions.length > 0) {
            for (IExtension ex : extensions) {
                if (ex.getNamespaceIdentifier().equals(Activator.PLUGIN_ID)) {
                    IConfigurationElement[] elems = ex.getConfigurationElements();
                    for (IConfigurationElement h : elems) {
                        if (h.getName().compareTo("sourcecodeview") == 0) {
                            Object obj = null;
                            try {
                                obj = h.createExecutableExtension("class");
                            
                                System.out.println(obj);
                            
                            } catch (CoreException e) {
                                e.printStackTrace();
                            }
                            if (obj instanceof MH_SourceCodeView) {
                                return (MH_SourceCodeView) obj;
                            }
                        }
                    }
                }
            }
        }
        return null;
    }
}

/**
 * {@link org.jtool.changereplayer.ui.ChangeHistoryEditor#EmptyEditorInput}
 */
class EmptyEditorInput implements IEditorInput {

    @Override
    public boolean exists() {
        return true;
    }

    @Override
    public ImageDescriptor getImageDescriptor() {
        return ImageDescriptor.getMissingImageDescriptor();
    }

    @Override
    public String getName() {
        return "Non-file";
    }

    @Override
    public String getToolTipText() {
        return getName();
    }

    @Override
    public IPersistableElement getPersistable() {
        return null;
    }

    @Override
    @SuppressWarnings({ "rawtypes", "unchecked" })
    public Object getAdapter(Class adapter) {
        return null;
    }
}
