package fse.eclipse.mergehelper.ui;

import java.util.List;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuCreator;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.IWorkbenchPage;
import org.jtool.changerecorder.util.Time;
import org.jtool.changereplayer.event.ViewChangedEvent;
import org.jtool.changereplayer.event.ViewEventSource;
import org.jtool.changereplayer.ui.HistoryView;
import org.jtool.changereplayer.ui.PackageExplorerView;
import org.jtool.changerepository.data.FileInfo;
import org.jtool.changerepository.event.RepositoryEventSource;
import org.jtool.changerepository.operation.CodeInsertedOperation;
import org.jtool.changerepository.operation.UnifiedOperation;

import fse.eclipse.mergehelper.Activator;
import fse.eclipse.mergehelper.element.BranchFileInfo;
import fse.eclipse.mergehelper.element.BranchInfo;
import fse.eclipse.mergehelper.element.BranchJavaElement;
import fse.eclipse.mergehelper.element.BranchRootInfo;
import fse.eclipse.mergehelper.element.ConflictInfo;
import fse.eclipse.mergehelper.element.MergePoint;
import fse.eclipse.mergehelper.element.MergeType;

public class MH_HistoryView extends HistoryView {
    public static final String ID = "ChangeHistory.view.MH_HistoryView";

    private static final ImageDescriptor mergeIcon = Activator.getImageDescriptor("icons/m_arrow.gif");
    private static final Image conflictIcon = Activator.getImage("icons/conflict.gif");

    private static MH_HistoryView instance;

    private BranchFileInfo bfInfo;
    private ConflictInfo cInfo;
    private int mergeIdx = -1;

    private Action elementButton, mergedButton;

    private static final String DefaultElementButtonText = "All Elements";
    private static final String OnlyConflictElementButtonText = "Conflict Only";
    private static final int IDX_COL = 0, UID_COL = 1, TIME_COL = 2, OP_COL = 3, ELEM_COL = 4;

    private static final String MERGE_MARK = "*"; // 文字化け原因

    public MH_HistoryView() {
        super();
        instance = this;
    }

    public static MH_HistoryView getInstance() {
        return instance;
    }

    public int getTableSize() {
        return operationTable.getItemCount();
    }

    public int getAMergeIndex() {
        return mergeIdx;
    }

    @Override
    public void createPartControl(Composite parent) {
        operationTable = new Table(parent, SWT.BORDER | SWT.SINGLE | SWT.CHECK | SWT.VIRTUAL | SWT.H_SCROLL | SWT.V_SCROLL);
        operationTable.setLinesVisible(true);
        operationTable.setHeaderVisible(true);

        TableColumn idxColumn = new TableColumn(operationTable, SWT.LEFT);
        idxColumn.setText("idx");
        idxColumn.setWidth(80);
        idxColumn.setResizable(true);

        TableColumn idColumn = new TableColumn(operationTable, SWT.LEFT);
        idColumn.setText("UID");
        idColumn.setWidth(80);
        idColumn.setResizable(true);

        TableColumn timeColumn = new TableColumn(operationTable, SWT.LEFT);
        timeColumn.setText("time");
        timeColumn.setWidth(10);
        timeColumn.setResizable(true);

        TableColumn typeColumn = new TableColumn(operationTable, SWT.LEFT);
        typeColumn.setText("operation");
        typeColumn.setWidth(200);
        typeColumn.setResizable(true);

        TableColumn detailsColumn = new TableColumn(operationTable, SWT.LEFT);
        detailsColumn.setText("element");
        detailsColumn.setWidth(300);
        detailsColumn.setResizable(true);

        final int MARGIN = 2;
        FormData opdata = new FormData();
        opdata.top = new FormAttachment(0, MARGIN);
        opdata.bottom = new FormAttachment(100, -MARGIN);
        opdata.left = new FormAttachment(0, MARGIN);
        opdata.right = new FormAttachment(100, -MARGIN);
        operationTable.setLayoutData(opdata);

        makeToolBarActions(parent);

        selectionListener = new OperationTableSelectionListener();
        operationTable.addSelectionListener(selectionListener);

        setOperationTable();
    }

    @Override
    protected PackageExplorerView getPackageExplorerView() {
        IWorkbenchPage workbenchPage = Activator.getWorkbenchPage();
        return (PackageExplorerView) workbenchPage.findView(MH_PackageExplorerView.ID);
    }

    @Override
    protected void setOperationTable() {
        PackageExplorerView pview = getPackageExplorerView();
        if (pview == null) {
            operationTable.removeAll();
            return;
        }

        FileInfo finfo = pview.getFileInfo();
        if (finfo == null) {
            operationTable.removeAll();
            return;
        }
        setOperationTable(finfo);

        int idx = pview.getEditor().getActiveView().getCurrentOperationIndex();
        operationTable.select(idx);
    }

    private void makeToolBarActions(Composite parent) {
        elementButton = new Action(DefaultElementButtonText, IAction.AS_DROP_DOWN_MENU) {
            @Override
            public void run() {
                sendViewEvent();
            }
        };
        elementButton.setEnabled(false);
        elementButton.setChecked(false);

        mergedButton = new Action("Only After Merge", IAction.AS_CHECK_BOX) {
            @Override
            public void run() {
                updateTableItemCheck(elementButton.getText(), mergedButton.isChecked());
                sendViewEvent();
            }
        };
        mergedButton.setImageDescriptor(mergeIcon);
        mergedButton.setEnabled(false);
        mergedButton.setChecked(true);

        IToolBarManager manager = getViewSite().getActionBars().getToolBarManager();
        manager.add(elementButton);
        manager.add(mergedButton);
    }

    @Override
    protected void setOperationTable(FileInfo fInfo) {
        List<UnifiedOperation> ops = fInfo.getOperations();
        if (ops.size() == 0) {
            return;
        }

        if (fileInfo == null || !fileInfo.equals(fInfo)) {
            fileInfo = fInfo;
            BranchRootInfo rootInfo = BranchRootInfo.getInstance();

            bfInfo = rootInfo.getBranchFileInfo(fInfo);
            cInfo = rootInfo.getConflictInfo();

            createTableItems(bfInfo);

            setupToolBarActions();

            operationTable.deselectAll();
            operationTable.update();

            sendViewEvent();
            if (!RefreshHolder.isRefreshed) {
                goToCheckedTop();
            } else {
                RefreshHolder.isRefreshed = false;
            }
        }
    }

    private void createTableItems(BranchFileInfo bfInfo) {
        operationTable.removeAll();
        List<UnifiedOperation> ops = fileInfo.getOperations();
        for (int i = 0; i < ops.size(); i++) {
            String id;
            String time;
            String text;
            String elemName;
            BranchJavaElement data;

            UnifiedOperation op = ops.get(i);
            if (CodeInsertedOperation.isCodeInsertedOperation(op)) {
                mergeIdx = i;
                id = MERGE_MARK;
                time = "";
                elemName = "";

                text = createAMergeText(op, bfInfo);
                data = null;
            } else {
                id = String.valueOf(op.getId());
                time = Time.toUsefulFormat(op.getTime());
                BranchJavaElement elem = bfInfo.getBranchJavaElement(op);
                if (elem != null) {
                    elemName = elem.getSimpleName();
                    data = elem;
                } else {
                    elemName = "";
                    data = null;
                }

                text = OperationRepresentation.createOperationTextualRepresentation(op);
            }

            TableItem item = new TableItem(operationTable, SWT.NONE);
            item.setText(IDX_COL, String.valueOf(i + 1));
            item.setText(UID_COL, id);
            item.setText(TIME_COL, time);
            item.setText(OP_COL, text);
            item.setText(ELEM_COL, elemName);
            item.setData(data);
            
            System.out.print(String.valueOf(i + 1));
            System.out.print(" " + id);
            System.out.print(" " + time);
            System.out.println(" " + elemName);
        }
    }

    private String createAMergeText(UnifiedOperation op, BranchFileInfo bfInfo) {
        MergeType type = bfInfo.getBranchInfo().getType();
        BranchInfo bInfo2 = BranchRootInfo.getInstance().getBranchInfo(MergeType.getAnotherType(type));
        List<BranchFileInfo> bfInfos2 = bInfo2.getAllBranchFileInfo();

        StringBuilder sb = new StringBuilder();
        sb.append("AMerge");
        for (BranchFileInfo bfInfo2 : bfInfos2) {
            if (bfInfo.equalsFileName(bfInfo2)) {
                MergePoint mPoint = BranchRootInfo.getInstance().getMergePoint();
                UnifiedOperation anotherOp = mPoint.getReferenceOperation(MergeType.getAnotherType(type));
                sb.append(":idx = ").append(anotherOp.indexOfFileInfo() + 1);
                return sb.toString();
            }
        }
        return sb.toString();
    }

    private void setupToolBarActions() {
        List<BranchJavaElement> elems = bfInfo.getAllBranchJavaElement();
        if (elems.size() > 0 && cInfo != null) {
            BranchJavaElement.sort(elems);

            elementButton.setMenuCreator(new MenuCreator(elems, cInfo.isConflict()));
            elementButton.setEnabled(true);

            mergedButton.setEnabled(true);
            mergedButton.setChecked(true);

            if (cInfo.isConflictFile(bfInfo.getFileInfo())) {
                int count = 0;
                for (BranchJavaElement elem : elems) {
                    if (cInfo.isConflictElement(elem)) {
                        count++;
                    }
                }
                if (count == 1) {
                    for (BranchJavaElement elem : elems) {
                        if (cInfo.isConflictElement(elem)) {
                            String name = elem.getSimpleName();
                            elementButton.setText(name);
                            updateTableItemCheck(name, true);
                            break;
                        }
                    }
                } else {
                    elementButton.setText(OnlyConflictElementButtonText);
                    updateTableItemCheck(OnlyConflictElementButtonText, true);
                }
            } else {
                elementButton.setText(DefaultElementButtonText);
                updateTableItemCheck(DefaultElementButtonText, true);
            }
        } else {
            elementButton.setEnabled(false);

            mergedButton.setEnabled(false);
            mergedButton.setChecked(false);
            elementButton.setText(DefaultElementButtonText);
            updateTableItemCheck(DefaultElementButtonText, false);
        }
    }

    @Override
    public void notify(ViewChangedEvent evt) {
        Object source = evt.getSource();

        if (source instanceof MH_SourceCodeView) {
            if (operationTable != null && operationTable.getItemCount() > 0) {
                MH_SourceCodeView view = (MH_SourceCodeView) source;
                setOperationTable(view.getFileInfo());
                if (!RefreshHolder.isRefreshed) {
                    goTo(view.getCurrentOperationIndex());
                } else {
                    goTo(RefreshHolder.idx);
                    // RefreshHolder.isRefreshed = false;
                    RefreshHolder.idx = -1;
                }
                return;
            }
        }
        super.notify(evt);
    }

    @Override
    public void setFocus() {
        operationTable.setFocus();
    };

    public void refresh() {
        bfInfo = null;
        fileInfo = null;
        cInfo = null;
        mergeIdx = -1;
        if (operationTable != null && !operationTable.isDisposed()) {
            RefreshHolder.isRefreshed = true;
            RefreshHolder.idx = currentOperationIndex;
            operationTable.removeAll();
        }
    }

    @Override
    public void dispose() {
        RepositoryEventSource.getInstance().removeEventListener(this);
        ViewEventSource.getInstance().removeEventListener(this);
        super.dispose();
    }

    @Override
    public void goTo(int idx) {
        if (fileInfo == null && idx < 0) {
            return;
        }

        if (operationTable.getItemCount() <= idx) {
            return;
        }

        currentOperationIndex = idx;
        operationTable.select(idx);
        reveal(idx);
    }

    private void goToCheckedTop() {
        int idx;
        TableItem[] items = operationTable.getItems();
        int length = items.length;
        for (idx = 0; idx < length; idx++) {
            if (items[idx].getChecked()) {
                break;
            }
        }
        goTo(idx);
    }

    private void sendViewEvent() {
        ViewChangedEvent event = new ViewChangedEvent(this);
        ViewEventSource.getInstance().fire(event);
    }

    private void updateTableItemCheck(String elemName, boolean isMergedOnly) {
        TableItem[] items = operationTable.getItems();
        int length = items.length;
        int i;
        if (isMergedOnly) {
            if (mergeIdx + 1 < length) {
                for (i = 0; i < mergeIdx; i++) {
                    items[i].setChecked(false);
                }
                if (mergeIdx != -1) {
                    i = mergeIdx;
                    items[i].setChecked(true);
                    i++;
                }
            } else {
                for (i = 0; i < length; i++) {
                    items[i].setChecked(false);
                }
                return;
            }
        } else {
            i = 0;
        }

        if (elemName.equals(DefaultElementButtonText)) {
            for (; i < length; i++) {
                items[i].setChecked(true);
            }
        } else if (elemName.equals(OnlyConflictElementButtonText)) {
            for (; i < length; i++) {
                TableItem item = items[i];
                Object data = item.getData();
                if (data != null && data instanceof BranchJavaElement) {
                    BranchJavaElement elem = (BranchJavaElement) data;
                    if (cInfo.isConflictElement(elem)) {
                        item.setChecked(true);
                    }
                }
            }
        } else {
            for (; i < length; i++) {
                TableItem item = items[i];
                String name = item.getText(ELEM_COL);
                if (name.equals(elemName) || i == mergeIdx) {
                    item.setChecked(true);
                } else {
                    item.setChecked(false);
                }
            }
        }
        goToCheckedTop();
    }

    private static class RefreshHolder {
        static boolean isRefreshed;
        static int idx;
    }

    private class OperationTableSelectionListener implements SelectionListener {

        OperationTableSelectionListener() {
        }

        @Override
        public void widgetDefaultSelected(SelectionEvent evt) {
        }

        @Override
        public void widgetSelected(SelectionEvent evt) {
            Table table = (Table) evt.getSource();
            int idx = (int) table.getSelectionIndex();

            goTo(idx);

            sendViewEvent();
        }
    }

    private class MenuCreator implements IMenuCreator {
        final List<BranchJavaElement> elems;
        final boolean isConflict;

        MenuCreator(List<BranchJavaElement> elems, boolean isConflict) {
            this.elems = elems;
            this.isConflict = isConflict;
        }

        @Override
        public Menu getMenu(Control parent) {
            Menu menu = new Menu(parent);
            MenuItem item0 = new MenuItem(menu, SWT.PUSH);
            item0.setText(DefaultElementButtonText);
            item0.addSelectionListener(new SelectionListener() {
                @Override
                public void widgetSelected(SelectionEvent event) {
                    MenuItem item = (MenuItem) event.widget;
                    String elemName = item.getText();
                    updateTableItemCheck(elemName, mergedButton.isChecked());
                    elementButton.setText(elemName);

                    sendViewEvent();
                }

                @Override
                public void widgetDefaultSelected(SelectionEvent event) {
                }
            });

            if (isConflict) {
                MenuItem item1 = new MenuItem(menu, SWT.PUSH);
                item1.setText(OnlyConflictElementButtonText);
                item1.addSelectionListener(new SelectionListener() {
                    @Override
                    public void widgetSelected(SelectionEvent event) {
                        MenuItem item = (MenuItem) event.widget;
                        String elemName = item.getText();
                        updateTableItemCheck(elemName, mergedButton.isChecked());
                        // elementButton.setText(indentText(elemName));
                        elementButton.setText(elemName);

                        sendViewEvent();
                    }

                    @Override
                    public void widgetDefaultSelected(SelectionEvent event) {
                    }
                });
            }

            for (BranchJavaElement elem : elems) {
                MenuItem item = new MenuItem(menu, SWT.PUSH);
                item.setText(elem.getSimpleName());
                if (cInfo != null && cInfo.isConflictElement(elem)) {
                    item.setImage(conflictIcon);
                }

                item.addSelectionListener(new SelectionListener() {
                    @Override
                    public void widgetSelected(SelectionEvent event) {
                        sendViewEvent();

                        MenuItem item = (MenuItem) event.widget;
                        String elemName = item.getText();
                        updateTableItemCheck(elemName, mergedButton.isChecked());
                        elementButton.setText(elemName);
                    }

                    @Override
                    public void widgetDefaultSelected(SelectionEvent arg0) {
                    }
                });
            }
            return menu;
        }

        @Override
        public Menu getMenu(Menu menu) {
            return null;
        }

        @Override
        public void dispose() {
        }
    }
}
