package fse.eclipse.mergehelper.ui.dialog;

import java.util.Map;
import java.util.Map.Entry;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.forms.widgets.ExpandableComposite;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.jtool.changerecorder.util.Time;
import org.jtool.changerepository.operation.UnifiedOperation;

import fse.eclipse.mergehelper.Activator;
import fse.eclipse.mergehelper.element.BranchInfo;
import fse.eclipse.mergehelper.element.BranchJavaElement;
import fse.eclipse.mergehelper.element.BranchRootInfo;
import fse.eclipse.mergehelper.element.ConflictInfo;
import fse.eclipse.mergehelper.element.MergePoint;
import fse.eclipse.mergehelper.element.MergeType;

public class DetectionResultDialog extends TitleAreaDialog implements IResultDialog {
    private static final String TITLE = "MergeHelper";
    private static final String MESSAGE_TITLE = "Detection Results";
    private static final Image icon = Activator.getImage("icons/cr.gif");
    private static final int WIDTH = 750, HEIGHT = 550;

    private FormToolkit toolkit;

    public DetectionResultDialog(Composite parent) {
        super(parent.getShell());
        setHelpAvailable(false);
    }

    @Override
    public void result() {
        super.open();
    }

    @Override
    protected Control createContents(Composite parent) {
        toolkit = new FormToolkit(parent.getDisplay());
        parent.addDisposeListener(new DisposeListener() {
            @Override
            public void widgetDisposed(DisposeEvent e) {
                toolkit.dispose();
            }
        });
        return super.createContents(parent);
    }

    @Override
    protected Control createDialogArea(Composite parent) {
        parent.getShell().setText(TITLE);
        setTitle(MESSAGE_TITLE);
        setTitleImage(icon);

        Composite container = toolkit.createComposite((Composite) super.createDialogArea(parent));
        GridDataFactory.fillDefaults().grab(true, true).applyTo(container);
        toolkit.paintBordersFor(container);
        GridLayoutFactory.swtDefaults().applyTo(container);

        SashForm form = new SashForm(container, SWT.VERTICAL | SWT.FILL);
        toolkit.adapt(form, true, true);
        form.setLayoutData(GridDataFactory.fillDefaults().grab(true, true).create());

        BranchRootInfo rootInfo = BranchRootInfo.getInstance();
        createOperationArea(form, rootInfo);
        if (rootInfo.getConflictInfo() != null) {
            createConflictElementArea(form, rootInfo);
            if (rootInfo.getMergePoint() != null) {
                createMergePointArea(form, rootInfo);
            }
        }

        parent.pack();
        return container;
    }

    @Override
    protected void createButtonsForButtonBar(Composite parent) {
        toolkit.adapt(parent, false, false);
        createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
    }

    @Override
    protected void buttonPressed(int buttonId) {
        okPressed();
    }

    private void createOperationArea(Composite container, BranchRootInfo rootInfo) {
        BranchInfo a_bInfo = rootInfo.getBranchInfo(MergeType.ACCEPT);
        BranchInfo j_bInfo = rootInfo.getBranchInfo(MergeType.JOIN);

        Section section = createSection(container);
        section.setText("Total Number of Edit Oerations");

        Composite subarea = createSubArea(section);
        StringBuilder sb = new StringBuilder();
        sb.append(a_bInfo.getName()).append(": ");
        sb.append(a_bInfo.getProjectInfo().getOperationNumber());

        Label alabel = new Label(subarea, SWT.NONE);
        alabel.setText(sb.toString());
        sb.delete(0, sb.length());

        sb.append(j_bInfo.getName()).append(": ");
        sb.append(j_bInfo.getProjectInfo().getOperationNumber());
        Label jlabel = new Label(subarea, SWT.NONE);
        jlabel.setText(sb.toString());
    }

    private void createConflictElementArea(Composite container, BranchRootInfo rootInfo) {
        BranchInfo a_bInfo = rootInfo.getBranchInfo(MergeType.ACCEPT);
        BranchInfo j_bInfo = rootInfo.getBranchInfo(MergeType.JOIN);

        Section section = createSection(container);
        section.setText("Conflict Elements");

        Composite subarea = createSubArea(section);

        Table elemTable = new Table(subarea, SWT.SINGLE | SWT.FULL_SELECTION | SWT.BORDER | SWT.VIRTUAL | SWT.H_SCROLL | SWT.V_SCROLL);
        elemTable.setLinesVisible(true);
        elemTable.setHeaderVisible(true);

        TableColumn elemColumn = new TableColumn(elemTable, SWT.LEFT);
        elemColumn.setResizable(true);

        TableColumn acceptPoint = new TableColumn(elemTable, SWT.LEFT);
        acceptPoint.setResizable(true);

        TableColumn joinPoint = new TableColumn(elemTable, SWT.LEFT);
        joinPoint.setResizable(true);

        Point size = new Point(500, 200);
        elemTable
                .setLayoutData(GridDataFactory.fillDefaults().grab(true, true).hint(size).minSize(size.x, size.y).align(SWT.FILL, SWT.FILL).create());

        String[] longestTexts = setElemTableItem(elemTable, rootInfo.getConflictInfo());
        packColumn(elemTable, longestTexts, new String[] { "Elements", a_bInfo.getName(), j_bInfo.getName() });
    }

    private void createMergePointArea(Composite container, BranchRootInfo rootInfo) {
        MergePoint mPoint = rootInfo.getMergePoint();

        Section section = createSection(container);
        section.setText("Artificial Merge Point");

        Composite subarea = createSubArea(section);
        if (mPoint != null) {
            BranchInfo a_bInfo = rootInfo.getBranchInfo(MergeType.ACCEPT);
            BranchInfo j_bInfo = rootInfo.getBranchInfo(MergeType.JOIN);
            UnifiedOperation a_op = mPoint.getReferenceOperation(MergeType.ACCEPT);
            UnifiedOperation j_op = mPoint.getReferenceOperation(MergeType.JOIN);

            StringBuilder sb = new StringBuilder();
            sb.append(a_bInfo.getName()).append(": ");
            sb.append(createTextOpRepresentation(a_op, -1)).append(" (");
            sb.append(mPoint.getReferenceIndexPercent(MergeType.ACCEPT)).append(")");

            Label alabel = new Label(subarea, SWT.NONE);
            alabel.setText(sb.toString());
            sb.delete(0, sb.length());

            sb.append(j_bInfo.getName()).append(": ");
            sb.append(createTextOpRepresentation(j_op, -1)).append(" (");
            sb.append(mPoint.getReferenceIndexPercent(MergeType.JOIN)).append(")");

            Label jlabel = new Label(subarea, SWT.NONE);
            jlabel.setText(sb.toString());
        } else {
            Label label = new Label(subarea, SWT.NONE);
            label.setText("no execute");
        }
    }

    private Section createSection(Composite container) {
        Composite area = new Composite(container, SWT.NONE);
        GridDataFactory.fillDefaults().grab(true, true).applyTo(area);
        GridLayoutFactory.swtDefaults().margins(0, 0).spacing(0, 0).applyTo(area);
        return toolkit.createSection(area, ExpandableComposite.TITLE_BAR | ExpandableComposite.CLIENT_INDENT);
    }

    private Composite createSubArea(Section section) {
        Composite subarea = toolkit.createComposite(section);
        section.setClient(subarea);
        GridLayoutFactory.fillDefaults().spacing(0, 0).extendedMargins(2, 2, 2, 2).applyTo(subarea);
        toolkit.paintBordersFor(subarea);
        GridDataFactory.fillDefaults().grab(true, true).applyTo(section);
        GridLayoutFactory.swtDefaults().applyTo(section);
        return subarea;
    }

    private String[] setElemTableItem(Table elemTable, ConflictInfo cInfo) {
        Map<BranchJavaElement, BranchJavaElement> confMap = cInfo.getConflictElementMap();
        StringBuilder sb = new StringBuilder();

        String[] longestTexts = new String[3];
        for (int i = 0; i < 3; i++) {
            longestTexts[i] = "";
        }

        for (Entry<BranchJavaElement, BranchJavaElement> entry : confMap.entrySet()) {
            BranchJavaElement a_elem = entry.getKey();
            BranchJavaElement j_elem = entry.getValue();

            UnifiedOperation a_op = a_elem.getOperations().get(0);
            UnifiedOperation j_op = j_elem.getOperations().get(0);

            sb.append(a_elem.getPackageName()).append("#").append(a_elem.getName());
            String elemText = sb.toString();
            String a_opText = createTextOpRepresentation(a_op, a_elem.getOperations().size());
            String j_opText = createTextOpRepresentation(j_op, j_elem.getOperations().size());

            if (longestTexts[0].length() < elemText.length()) {
                longestTexts[0] = elemText;
            }
            if (longestTexts[1].length() < a_opText.length()) {
                longestTexts[1] = a_opText;
            }
            if (longestTexts[2].length() < j_opText.length()) {
                longestTexts[2] = j_opText;
            }

            TableItem item = new TableItem(elemTable, SWT.NONE);
            item.setText(0, elemText);
            item.setText(1, a_opText);
            item.setText(2, j_opText);

            sb.delete(0, sb.length());
        }
        return longestTexts;
    }

    private String createTextOpRepresentation(UnifiedOperation op, int size) {
        StringBuilder sb = new StringBuilder();
        sb.append("idx:").append(op.indexOfProjectInfo()).append("  UID:");
        sb.append(op.getId());
        if (size >= 0) {
            sb.append(" size:").append(size);
        }
        sb.append(" [");
        sb.append(Time.toUsefulFormat(op.getTime())).append("]");
        return sb.toString();
    }

    private void packColumn(Table elemTable, String[] longestTexts, String[] cnames) {
        if (longestTexts.length != cnames.length) {
            throw new Error();
        }

        int length = longestTexts.length;
        TableColumn[] columns = elemTable.getColumns();
        for (int i = 0; i < length; i++) {
            TableColumn column = columns[i];
            column.setText(longestTexts[i]);
            column.pack();
            column.setText(cnames[i]);
        }
    }

    @Override
    protected boolean isResizable() {
        return true;
    }

    @Override
    protected Point getInitialSize() {
        return new Point(WIDTH, HEIGHT);
    }
}
