package fse.eclipse.mergehelper.ui.dialog;

import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.forms.widgets.ExpandableComposite;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;

import fse.eclipse.mergehelper.Activator;
import fse.eclipse.mergehelper.detector.DetectOption;
import fse.eclipse.mergehelper.element.BranchRootInfo;
import fse.eclipse.mergehelper.element.MergeType;
import fse.eclipse.mergehelper.util.ProjectUtil;

public class ProjectSelectDialog extends TitleAreaDialog {
    private static final String TITLE = "MergeHelper";
    private static final String MESSAGE_TITLE = "Please select a project and its conflicting braches";
    private static final Image icon = Activator.getImage("icons/cr.gif");
    private static final int WIDTH = 600, HEIGHT = 750;

    private FormToolkit toolkit;
    private Table projectTable, acceptTable, joinTable;
    private Button isSliceBt, isSwapBt, isAMergeBt, okBt;

    private IProject project;

    public ProjectSelectDialog(Composite parent) {
        super(parent.getShell());
        setHelpAvailable(false);
    }

    @Override
    protected Control createContents(Composite parent) {
        toolkit = new FormToolkit(parent.getDisplay());
        parent.addDisposeListener(new DisposeListener() {
            @Override
            public void widgetDisposed(DisposeEvent e) {
                toolkit.dispose();
            }
        });
        return super.createContents(parent);
    }

    @Override
    protected Control createDialogArea(Composite parent) {
        parent.getShell().setText(TITLE);
        setTitle(MESSAGE_TITLE);
        setTitleImage(icon);

        Composite container = toolkit.createComposite((Composite) super.createDialogArea(parent));
        GridDataFactory.fillDefaults().grab(true, true).applyTo(container);
        toolkit.paintBordersFor(container);
        GridLayoutFactory.swtDefaults().applyTo(container);

        SashForm form = new SashForm(container, SWT.VERTICAL | SWT.FILL);
        toolkit.adapt(form, true, true);
        form.setLayoutData(GridDataFactory.fillDefaults().grab(true, true).create());

        createProjectArea(form);
        createAcceptBranchArea(form);
        createJoinBranchArea(form);
        createOptionArea(form);

        return container;
    }

    @Override
    protected void createButtonsForButtonBar(Composite parent) {
        toolkit.adapt(parent, false, false);
        okBt = createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
        okBt.setEnabled(false);
        createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);

        restore();
    }

    @Override
    protected void buttonPressed(int buttonId) {
        if (buttonId == IDialogConstants.OK_ID) {
            okPressed();
        } else if (buttonId == IDialogConstants.CANCEL_ID) {
            cancelPressed();
        }
    }

    @Override
    protected void okPressed() {
        String acceptName = acceptTable.getSelection()[0].getText(0);
        String joinName = joinTable.getSelection()[0].getText(0);

        BranchRootInfo.create(project, acceptName, joinName);

        ConflictDetectingDialog nextDialog = new ConflictDetectingDialog(new ProgressMonitorDialog(null));
        DetectOption option = DetectOption.getInstance();
        option.setSliceFlag(isSliceBt.getSelection());
        option.setSwapFlag(isSwapBt.getSelection());
        option.setAMergeFlag(isAMergeBt.getSelection());

        super.okPressed();
        nextDialog.open();
    }

    private void createProjectArea(Composite container) {
        Section section = createSection(container);
        section.setText("Project");

        Composite subarea = createSubArea(section);

        projectTable = new Table(subarea, SWT.SINGLE | SWT.FULL_SELECTION);
        Point size = new Point(WIDTH, HEIGHT);
        projectTable.setLayoutData(GridDataFactory.fillDefaults().grab(true, true).hint(size).minSize(size.x, 0).align(SWT.FILL, SWT.FILL).create());

        IProject[] ps = ProjectUtil.getProjects();
        for (IProject p : ps) {
            if (ProjectUtil.hasMultipleBranchHistory(p)) {
                TableItem item = new TableItem(projectTable, SWT.NONE);
                item.setText(p.getName());
            }
        }

        projectTable.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetSelected(SelectionEvent event) {
                okBt.setEnabled(false);

                String projectName = projectTable.getSelection()[0].getText(0);
                project = ProjectUtil.convertIProject(projectName);

                List<String> nameList = ProjectUtil.getBranchHistoryNameList(project);
                setBranchNameTableItem(acceptTable, nameList);
            }

            @Override
            public void widgetDefaultSelected(SelectionEvent event) {
            }
        });
    }

    private void createAcceptBranchArea(Composite container) {
        Section section = createSection(container);
        section.setText("Branch [Accept]");

        Composite subarea = createSubArea(section);

        acceptTable = new Table(subarea, SWT.SINGLE | SWT.FULL_SELECTION);
        Point size = new Point(WIDTH, HEIGHT);
        acceptTable.setLayoutData(GridDataFactory.fillDefaults().grab(true, true).hint(size).minSize(size.x, 0).align(SWT.FILL, SWT.FILL).create());
        acceptTable.setSize(size.x, 0);

        acceptTable.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetSelected(SelectionEvent event) {
                okBt.setEnabled(false);

                List<String> nameList = ProjectUtil.getBranchHistoryNameList(project);
                nameList.remove(acceptTable.getSelection()[0].getText(0));
                setBranchNameTableItem(joinTable, nameList);
            }

            @Override
            public void widgetDefaultSelected(SelectionEvent event) {
            }
        });
    }

    private void createJoinBranchArea(Composite container) {
        Section section = createSection(container);
        section.setText("Branch [Join]");

        Composite subarea = createSubArea(section);

        joinTable = new Table(subarea, SWT.SINGLE | SWT.FULL_SELECTION);
        Point size = new Point(WIDTH, HEIGHT);
        joinTable.setLayoutData(GridDataFactory.fillDefaults().grab(true, true).hint(size).minSize(size.x, 0).align(SWT.FILL, SWT.FILL).create());

        joinTable.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetSelected(SelectionEvent event) {
                okBt.setEnabled(true);
            }

            @Override
            public void widgetDefaultSelected(SelectionEvent event) {
            }
        });
    }

    private void createOptionArea(Composite container) {
        Section section = createSection(container);
        section.setText("Options");

        Composite subarea = createSubArea(section);
        DetectOption option = DetectOption.getInstance();

        isSliceBt = new Button(subarea, SWT.CHECK);
        isSliceBt.setText("operation history slicing");
        isSliceBt.setSelection(option.isSlice());

        isSwapBt = new Button(subarea, SWT.CHECK);
        isSwapBt.setText("operation swapping");
        isSwapBt.setSelection(option.isSwap());

        isAMergeBt = new Button(subarea, SWT.CHECK);
        isAMergeBt.setText("artificial merging");
        isAMergeBt.setSelection(option.isAMerge());

        // isOpNormalizeBt = new Button(subarea, SWT.CHECK);
        // isOpNormalizeBt.setText("operation normalize");
        // isOpNormalizeBt.setSelection(option.isOpNormalize());

        subarea.pack();
    }

    private Section createSection(Composite container) {
        Composite area = new Composite(container, SWT.NONE);
        GridDataFactory.fillDefaults().grab(true, true).applyTo(area);
        GridLayoutFactory.swtDefaults().margins(0, 0).spacing(0, 0).applyTo(area);
        return toolkit.createSection(area, ExpandableComposite.TITLE_BAR | ExpandableComposite.CLIENT_INDENT);
    }

    private Composite createSubArea(Section section) {
        Composite subarea = toolkit.createComposite(section);
        section.setClient(subarea);
        GridLayoutFactory.fillDefaults().spacing(0, 0).extendedMargins(2, 2, 2, 2).applyTo(subarea);
        toolkit.paintBordersFor(subarea);
        GridDataFactory.fillDefaults().grab(true, true).applyTo(section);
        GridLayoutFactory.swtDefaults().applyTo(section);
        return subarea;
    }

    private void setBranchNameTableItem(Table table, List<String> nameList) {
        table.removeAll();

        TableItem item;
        for (String name : nameList) {
            item = new TableItem(table, SWT.NONE);
            item.setText(name);
        }
    }

    private void restore() {
        BranchRootInfo rootInfo = BranchRootInfo.getInstance();
        if (rootInfo != null) {
            List<String> nameList = null;
            String pName = rootInfo.getProject().getName();
            TableItem[] items = projectTable.getItems();
            int length = items.length;
            for (int i = 0; i < length; i++) {
                TableItem item = items[i];
                if (item.getText().equals(pName)) {
                    projectTable.setSelection(i);
                    project = ProjectUtil.convertIProject(pName);
                    nameList = ProjectUtil.getBranchHistoryNameList(project);
                    break;
                }
            }

            if (nameList == null) {
                return;
            }
            setBranchNameTableItem(acceptTable, nameList);
            String a_name = rootInfo.getBranchName(MergeType.ACCEPT);
            items = acceptTable.getItems();
            length = items.length;
            for (int i = 0; i < length; i++) {
                TableItem item = items[i];
                if (item.getText().toString().equals(a_name)) {
                    acceptTable.setSelection(i);
                    nameList.remove(a_name);
                    break;
                }
            }

            setBranchNameTableItem(joinTable, nameList);
            String j_name = rootInfo.getBranchName(MergeType.JOIN);
            items = joinTable.getItems();
            length = items.length;
            for (int i = 0; i < length; i++) {
                TableItem item = items[i];
                if (item.getText().toString().equals(j_name)) {
                    joinTable.setSelection(i);
                    okBt.setEnabled(true);
                    break;
                }
            }
        }
    }

    @Override
    protected boolean isResizable() {
        return true;
    }

    @Override
    protected Point getInitialSize() {
        return new Point(WIDTH, HEIGHT);
    }
}
